@@ -3,9 +3,23 @@
 #include <string.h>
 #include <inttypes.h>
 #include <sys/stat.h>
+#include <math.h>
 
 #include "hmmguts.h"
 
+double kahan_accum(double accum, double *pcompensation, double x)
+{
+  /*
+   * Return the new sum, and update the compensation.
+   * This function exists to deal with rounding errors.
+   * See Kahan summation.
+   */
+  double y = x - *pcompensation;
+  double t = accum + y;
+  *pcompensation = (t - accum) - y;
+  return t;
+}
+
 int fsafeclose(FILE *f)
 {
   if (f)
@@ -329,6 +343,194 @@ int posterior_alldisk(int nstates,
   return 0;
 }
 
+int state_expectations_alldisk(int nstates, double *expectations, FILE *fi_d)
+{
+  /*
+   * Compute the expected amount of time spent in each state.
+   */
+  int i;
+  double *compensations = malloc(nstates*sizeof(double));
+  double *probabilities = malloc(nstates*sizeof(double));
+  for (i=0; i<nstates; ++i)
+  {
+    compensations[i] = 0.0;
+  }
+  while (fread(probabilities, sizeof(double), nstates, fi_d))
+  {
+    for (i=0; i<nstates; i++)
+    {
+      expectations[i] = kahan_accum(
+          expectations[i], compensations+i, probabilities[i]);
+    }
+  }
+  free(compensations);
+  free(probabilities);
+  return 0;
+}
+
+int transition_expectations_alldisk(int nstates, const double *trans,
+    double *expectations, FILE *fi_l, FILE *fi_f, FILE *fi_b)
+{
+  /*
+   * Implementation is from my tested ExternalHMM python code.
+   */
+  int i;
+  size_t nbytes;
+  int result;
+  /* seek to near the end of the backward file */
+  result = fseek(fi_b, -nstates*sizeof(double), SEEK_END);
+  if (result)
+  {
+    fprintf(stderr, "seek error\n");
+    return -1;
+  }
+  /* initialize compensations for kahan summation of transitions */
+  double *compensations = malloc(nstates*nstates*sizeof(double));
+  for (i=0; i<nstates*nstates; ++i)
+  {
+    compensations[i] = 0.0;
+  }
+  /* initialize some states */
+  double *l_old = malloc(nstates*sizeof(double));
+  double *f_old = malloc(nstates*sizeof(double));
+  double *b_old = malloc(nstates*sizeof(double));
+  double *l_new = malloc(nstates*sizeof(double));
+  double *f_new = malloc(nstates*sizeof(double));
+  double *b_new = malloc(nstates*sizeof(double));
+  double *tmp;
+  int source, sink;
+  int index;
+  double x;
+  int count = 0;
+  do
+  {
+    nbytes = fread(l_new, sizeof(double), nstates, fi_l);
+    nbytes = fread(f_new, sizeof(double), nstates, fi_f);
+    nbytes = fread(b_new, sizeof(double), nstates, fi_b);
+    if (count)
+    {
+      for (source=0; source<nstates; ++source)
+      {
+        for (sink=0; sink<nstates; ++sink)
+        {
+          index = source * nstates + sink;
+          x = f_old[source] * trans[index] * l_new[sink] * b_new[sink];
+          expectations[index] = kahan_accum(
+              expectations[index], compensations+index, x);
+        }
+      }
+    }
+    tmp = l_old; l_old = l_new; l_new = tmp;
+    tmp = f_old; f_old = f_new; f_new = tmp;
+    tmp = b_old; b_old = b_new; b_new = tmp;
+    ++count;
+    /* go back a bit */
+    result = fseek(fi_b, -2*nstates*sizeof(double), SEEK_CUR);
+  } while (!result);
+  free(l_old);
+  free(f_old);
+  free(b_old);
+  free(l_new);
+  free(f_new);
+  free(b_new);
+  free(compensations);
+  return 0;
+}
+
+int emission_expectations_alldisk(int nstates, int nalpha,
+    double *expectations, FILE *fi_v, FILE *fi_d)
+{
+  /*
+   * Compute the expected emissions from each state.
+   * @param nstates: the number of hidden states
+   * @param nalpha: the size of the alphabet, at most 256
+   */
+  int errcode = 0;
+  int i;
+  /* initialize compensations for kahan summation */
+  double *compensations = malloc(nstates*nalpha*sizeof(double));
+  for (i=0; i<nstates*nalpha; ++i)
+  {
+    compensations[i] = 0.0;
+  }
+  /* accumulate the expectations */
+  int index;
+  int nbytes;
+  unsigned char emission;
+  double *probabilities = malloc(nstates*sizeof(double));
+  while (fread(probabilities, sizeof(double), nstates, fi_d))
+  {
+    nbytes = fread(&emission, sizeof(unsigned char), 1, fi_v);
+    if (emission >= nalpha)
+    {
+      fprintf(stderr, "an observation is out of range\n");
+      errcode = -1; goto end;
+    }
+    for (i=0; i<nstates; ++i)
+    {
+      index = i * nalpha + emission;
+      expectations[index] = kahan_accum(
+          expectations[index], compensations+index, probabilities[i]);
+    }
+  }
+end:
+  free(compensations);
+  free(probabilities);
+  return errcode;
+}
+
+int finite_alphabet_likelihoods_alldisk(int nstates, int nalpha,
+    const double *emissions, FILE *fi_v, FILE *fo_l)
+{
+  /*
+   * Write likelihoods given observations.
+   * This only works when the alphabet of observations is finite and small.
+   * In other situations the likelihoods must be computed by the user.
+   * Note that the observation 127 is reserved for missing data.
+   * @param nstates: the number of hidden states
+   * @param nalpha: the size of the alphabet, less than 127
+   * @param emissions: matrix with size (nstates, nalpha)
+   */
+  int errcode = 0;
+  int i;
+  int index;
+  unsigned char emission;
+  double one = 1.0;
+  while (fread(&emission, sizeof(unsigned char), 1, fi_v))
+  {
+    if (emission == 127) {
+      for (i=0; i<nstates; ++i) {
+        fwrite(&one, sizeof(double), 1, fo_l);
+      }
+    } else {
+      if (emission >= nalpha) {
+        fprintf(stderr, "an observation is out of range\n");
+        errcode = -1; goto end;
+      }
+      for (i=0; i<nstates; ++i) {
+        index = i * nalpha + emission;
+        fwrite(emissions+index, sizeof(double), 1, fo_l);
+      }
+    }
+  }
+end:
+  return errcode;
+}
+
+int sequence_log_likelihood_alldisk(double *p, FILE *fi_s)
+{
+  double accum = 0.0;
+  double compensation = 0.0;
+  double s;
+  while (fread(&s, sizeof(double), 1, fi_s))
+  {
+    accum = kahan_accum(accum, &compensation, log(s));
+  }
+  *p = accum;
+  return 0;
+}
+
+
 int forward_somedisk(const struct TM *ptm, FILE *fin_l,
     double *f_big, double *s_big)
 {
@@ -607,6 +809,132 @@ end:
   return errcode;
 }
 
+int transition_expectations_nodisk(int nstates, int nobs,
+    const double *trans, double *expectations, 
+    const double *l_big, const double *f_big, const double *b_big)
+{
+  int i;
+  /* initialize compensations for kahan summation of transitions */
+  double *compensations = malloc(nstates*nstates*sizeof(double));
+  for (i=0; i<nstates*nstates; ++i)
+  {
+    compensations[i] = 0.0;
+  }
+  /* initialize some states */
+  const double *f_prev = f_big;
+  const double *l_curr = l_big + nstates;
+  const double *b_curr = b_big + nstates * (nobs - 2);
+  int source, sink;
+  int index;
+  double x;
+  for (i=0; i<nobs-1; i++)
+  {
+    for (source=0; source<nstates; ++source)
+    {
+      for (sink=0; sink<nstates; ++sink)
+      {
+        index = source * nstates + sink;
+        x = f_prev[source] * trans[index] * l_curr[sink] * b_curr[sink];
+        expectations[index] = kahan_accum(
+            expectations[index], compensations+index, x);
+      }
+    }
+    f_prev += nstates;
+    l_curr += nstates;
+    b_curr -= nstates;
+  }
+  free(compensations);
+  return 0;
+}
+
+int emission_expectations_nodisk(int nstates, int nalpha, int nobs,
+    double *expectations, const unsigned char *v_big, const double *d_big)
+{
+  int errcode = 0;
+  int i;
+  /* initialize compensations for kahan summation */
+  double *compensations = malloc(nstates*nalpha*sizeof(double));
+  for (i=0; i<nstates*nalpha; ++i)
+  {
+    compensations[i] = 0.0;
+  }
+  /* accumulate the expectations */
+  unsigned char emission;
+  int obs_index;
+  int expectations_index;
+  int posterior_index;
+  for (obs_index=0; obs_index < nobs; ++obs_index)
+  {
+    emission = v_big[obs_index];
+    if (emission == 127) {
+      /* missing data does not inform the emission expectations */
+      continue;
+    }
+    if (emission >= nalpha) {
+      fprintf(stderr, "an observation is out of range\n");
+      errcode = -1; goto end;
+    }
+    for (i=0; i<nstates; ++i) {
+      expectations_index = i * nalpha + emission;
+      posterior_index = obs_index * nstates + i;
+      expectations[expectations_index] = kahan_accum(
+          expectations[expectations_index],
+          compensations+expectations_index, d_big[posterior_index]);
+    }
+  }
+end:
+  free(compensations);
+  return errcode;
+}
+
+int finite_alphabet_likelihoods_nodisk(int nstates, int nalpha, int nobs,
+    const double *emissions, 
+    const unsigned char *v_big, double *l_big)
+{
+  int errcode = 0;
+  int obs_index;
+  int i;
+  int emissions_index;
+  int likelihoods_index;
+  unsigned char emission;
+  for (obs_index=0; obs_index<nobs; ++obs_index)
+  {
+    emission = v_big[obs_index];
+    if (emission == 127) {
+      for (i=0; i<nstates; ++i) {
+        likelihoods_index = obs_index * nstates + i;
+        l_big[likelihoods_index] = 1.0;
+      }
+    } else {
+      if (emission >= nalpha) {
+        fprintf(stderr, "an observation is out of range\n");
+        errcode = -1; goto end;
+      }
+      for (i=0; i<nstates; ++i) {
+        emissions_index = i * nalpha + emission;
+        likelihoods_index = obs_index * nstates + i;
+        l_big[likelihoods_index] = emissions[emissions_index];
+      }
+    }
+  }
+end:
+  return errcode;
+}
+
+int sequence_log_likelihood_nodisk(double *p, int nobs, const double *s_big)
+{
+  double accum = 0.0;
+  double compensation = 0.0;
+  int i;
+  for (i=0; i<nobs; i++)
+  {
+    accum = kahan_accum(accum, &compensation, log(s_big[i]));
+  }
+  *p = accum;
+  return 0;
+}
+
+
 
 int do_fwdbwd_somedisk(const struct TM *ptm,
     const char *likelihoods_name, const char *posterior_name)
@@ -745,3 +1073,125 @@ end:
   fsafeclose(fout_d);
   return errcode;
 }
+
+int do_state_expectations(int nstates,
+    double *expectations, const char *posterior_name)
+{
+  int errcode = 0;
+  FILE *fin_d = NULL;
+  if (!(fin_d = fopen(posterior_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the posterior vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  state_expectations_alldisk(nstates, expectations, fin_d);
+end:
+  fsafeclose(fin_d);
+  return errcode;
+}
+
+int do_transition_expectations(int nstates, const double *trans,
+    double *expectations,
+    const char *likelihoods_name, const char *forward_name,
+    const char *backward_name)
+{
+  int errcode = 0;
+  FILE *fin_l = NULL;
+  FILE *fin_f = NULL;
+  FILE *fin_b = NULL;
+  if (!(fin_l = fopen(likelihoods_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the likelihoods vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (!(fin_f = fopen(forward_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the forward factor file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (!(fin_b = fopen(backward_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the backward vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  transition_expectations_alldisk(
+      nstates, trans, expectations, fin_l, fin_f, fin_b);
+end:
+  fsafeclose(fin_l);
+  fsafeclose(fin_f);
+  fsafeclose(fin_b);
+  return errcode;
+}
+
+int do_emission_expectations(int nstates, int nalpha, double *expectations,
+    const char *observation_name, const char *posterior_name)
+{
+  int errcode = 0;
+  FILE *fin_v = NULL;
+  FILE *fin_d = NULL;
+  if (!(fin_v = fopen(observation_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the observation vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (!(fin_d = fopen(posterior_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the posterior vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (emission_expectations_alldisk(
+      nstates, nalpha, expectations, fin_v, fin_d))
+  {
+    errcode = -1; goto end;
+  }
+end:
+  fsafeclose(fin_v);
+  fsafeclose(fin_d);
+  return errcode;
+}
+
+int do_finite_alphabet_likelihoods(int nstates, int nalpha,
+    const double *emissions,
+    const char *observation_name, const char *likelihood_name)
+{
+  int errcode = 0;
+  FILE *fin_v = NULL;
+  FILE *fout_l = NULL;
+  if (!(fin_v = fopen(observation_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the observation vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (!(fout_l = fopen(likelihood_name, "wb")))
+  {
+    fprintf(stderr, "failed to open the likelihood vector file for writing\n");
+    errcode = -1; goto end;
+  }
+  if (finite_alphabet_likelihoods_alldisk(
+      nstates, nalpha, emissions, fin_v, fout_l))
+  {
+    errcode = -1; goto end;
+  }
+end:
+  fsafeclose(fin_v);
+  fsafeclose(fout_l);
+  return errcode;
+}
+
+int do_sequence_log_likelihood(double *p, const char *scaling_name)
+{
+  int errcode = 0;
+  FILE *fin_s = NULL;
+  if (!(fin_s = fopen(scaling_name, "rb")))
+  {
+    fprintf(stderr, "failed to open the scaling vector file for reading\n");
+    errcode = -1; goto end;
+  }
+  if (sequence_log_likelihood_alldisk(p, fin_s))
+  {
+    errcode = -1; goto end;
+  }
+end:
+  fsafeclose(fin_s);
+  return errcode;
+}
