@@ -8,7 +8,13 @@ import argparse
 import numpy as np
 
 
-class DurbinDemo:
+class Demo:
+
+    def get_nstates(self):
+        return len(self.get_distribution())
+
+
+class DurbinDemo(Demo):
     """
     From the book Biological Sequence Analysis.
     """
@@ -69,12 +75,15 @@ class DurbinDemo:
         return np.array([[d[obs] for d in distns] for obs in observations])
 
 
-class EddyDemo:
+class EddyDemo(Demo):
     """
     NATURE BIOTECHNOLOGY VOLUME 22 NUMBER 10 OCTOBER 2004
     ftp://selab.janelia.org/pub/publications/Eddy-ATG4/Eddy-ATG4-reprint.pdf
     """
 
+    def get_nalpha(self):
+        return 5
+
     def get_distribution(self):
         p_E = 1.0
         p_5 = 0.0
@@ -90,7 +99,7 @@ class EddyDemo:
             [0.0, 0.0, 0.0, 1.0]])
 
     def get_likelihoods(self):
-        observations = 'CTTCATGTGAAAGCAGACGTAAGTCAx'
+        observations = self.get_observations()
         d_E = {'A':0.25, 'C':0.25, 'G':0.25, 'T':0.25, 'x':0.0}
         d_5 = {'A':0.05, 'C':0.00, 'G':0.95, 'T':0.00, 'x':0.0}
         d_I = {'A':0.40, 'C':0.10, 'G':0.10, 'T':0.40, 'x':0.0}
@@ -100,6 +109,14 @@ class EddyDemo:
             assert abs(1.0 - sum(distn.values())) < 1e-10
         return np.array([[d[obs] for d in distns] for obs in observations])
 
+    def get_observations(self):
+        return 'CTTCATGTGAAAGCAGACGTAAGTCAx'
+
+    def get_int8_observations(self):
+        observations = self.get_observations()
+        c_to_int8 = {'A':0, 'C':1, 'G':2, 'T':3, 'x':4}
+        return np.array([c_to_int8[x] for x in observations], dtype=np.int8)
+
     def get_expected_scaling(self):
         """
         This is for regression testing.
@@ -233,12 +250,32 @@ class EddyDemo:
             [4.44444444e+00, 2.19003771e-01, 1.97103394e-01, 0.00000000e+00],
             [4.00000000e+00, 3.15365430e-01, 2.83828887e-01, 0.00000000e+00]])
 
+    def get_expected_state_expectations(self):
+        return np.array([17.69120349, 1.0, 7.30879651, 1.0])
 
-class SmithDemo:
+    def get_expected_transition_expectations(self):
+        return np.array([
+            [ 16.69120349,   1.,           0.,           0.        ],
+            [  0.,           0.,           1.,           0.        ],
+            [  0.,           0.,           6.30879651,   1.        ],
+            [  0.,           0.,           0.,           0.        ]])
+
+    def get_expected_emission_expectations(self):
+        return np.array([
+            [ 5.95486273,  3.64338648,  3.82944037,  4.2635139,   0.  ],
+            [ 0.02617222,  0.,          0.97382778,  0.,          0.  ],
+            [ 3.01896505,  1.35661352,  1.19673184,  1.7364861,   0.  ],
+            [ 0.,          0.,          0.,          0.,          1.  ]])
+
+
+class SmithDemo(Demo):
     """
     http://www.cs.cmu.edu/~nasmith/papers/smith.tut04a.pdf
     """
 
+    def get_nalpha(self):
+        return 3
+
     def get_distribution(self):
         p_1 = 1.0
         p_2 = 0.0
@@ -252,7 +289,7 @@ class SmithDemo:
                 [0, 0, 1.0]])
 
     def get_likelihoods(self):
-        observations = 'XXXXx'
+        observations = self.get_observations()
         d_1 = {'X':7/8.0, 'Y':1/8.0, 'x':0.00}
         d_2 = {'X':1/16.0, 'Y':15/16.0, 'x':0.00}
         d_x = {'X':0.00, 'Y':0.00, 'x':1.00}
@@ -261,6 +298,14 @@ class SmithDemo:
             assert abs(1.0 - sum(distn.values())) < 1e-10
         return np.array([[d[obs] for d in distns] for obs in observations])
 
+    def get_observations(self):
+        return 'XXXXx'
+
+    def get_int8_observations(self):
+        observations = self.get_observations()
+        c_to_int8 = {'X':0, 'Y':1, 'x':2}
+        return np.array([c_to_int8[x] for x in observations], dtype=np.int8)
+
     def get_expected_scaling(self):
         """
         This is for regression testing.
@@ -304,3 +349,18 @@ class SmithDemo:
             [3.397305022458146, 7.839934667211106, 0.000000000000000],
             [4.730093915884034, 10.29644752960392, 0.000000000000000],
             [1.142857142857143, 2.548912092399230, 0.000000000000000]])
+
+    def get_expected_state_expectations(self):
+        return np.array([2.87790935, 1.12209065, 1.0])
+
+    def get_expected_transition_expectations(self):
+        return np.array([
+            [ 1.15761535,  1.06574112,  0.65455288],
+            [ 0.720294,    0.05634953,  0.34544712],
+            [ 0.,          0.,          0.        ]])
+
+    def get_expected_emission_expectations(self):
+        return np.array([
+            [ 2.87790935,  0.,          0.        ],
+            [ 1.12209065,  0.,          0.        ],
+            [ 0.,          0.,          1.        ]])
